######################### 
## VAF_T, VAF_N, pvalue are from the pipeline
## qlim = quantile limit of the statistics to avoid outliers
## 
## n0 = percent of points near the origin where the null is enforced
## fc.cut = threshold of the fold change. If fc<cut then fdr=1
## m = moderating constant to compute VAFs
##

fdrmut <-
function (t1,t2,n1,n2, pvalue, nr =64, qlim= 0.999, 
     fc.cut=2, n0=0.05, m=2, smooth = 0.9, verb = TRUE,  ...) 
{
    vcat = function(...) if (verb) 
        cat(...)
   if ((smooth < 0.01) | (smooth > 0.99)) {
        smooth = min(max(smooth, 0.01), 0.99)
        warning("smooth must be between 0.01 and 0.99 - reset to ", 
            smooth)
    }
   # more moderated VAFs
   vaft = (t2+m)/(t1+t2+2*m)
   vafn = (n2+m)/(n1+n2+2*m)
   fc = vaft/vafn
   logp= -10*log10(Chi2)

    ## treat the negative logfc as null pattern
    pick = fc <1
    x = abs(log2(fc)); xnull= -log2(fc[pick])
    y = logp; ynull = logp[pick]

    xbreaks = MidBreaks(x[x<quantile(x,qlim)], nr)
    ybreaks = MidBreaks(y[y<quantile(y,qlim)], nr)
    xmid = xbreaks[-1] - diff(xbreaks)/2
    ymid = ybreaks[-1] - diff(ybreaks)/2

    cut.stat.x = cut(x, xbreaks, include.lowest = TRUE)
    cut.stat.y = cut(y, ybreaks, include.lowest = TRUE)
    cut.null.x = cut(xnull, xbreaks, include.lowest = TRUE)
    cut.null.y = cut(ynull, ybreaks, include.lowest = TRUE)
    tab.stat = table(cut.stat.x, cut.stat.y)
    tab.null = table(cut.null.x, cut.null.y)
 
    ### need to equalize the (0,0) point
    N0 = n0*nr; N0 = ceiling(N0)
    tab.stat[1:N0,1:N0]= tab.null[1:N0,1:N0]= 
        (tab.stat[1:N0,1:N0]+ tab.null[1:N0,1:N0])/2   

    df.smooth = (1 - smooth) * (nr - 1)^2
    b = smooth2d.yn(tab.null, tab.stat, df = df.smooth)
    f0fz = b$fit/(1 - b$fit)
       fdr = ifelse(f0fz>1,1,f0fz)
    dimnames(fdr) = dimnames(tab.stat)

    ## indiv fdr values, a bit slow, cut the fc to reduce computation
    pick = fc> fc.cut
    ifdr = rep(1,length(fc))
    ifdr[pick] = approx2d(xmid, ymid, fdr, x[pick], y[pick])
    out = data.frame(VAF_T=vaft,VAF_N=vafn, pvalue=pvalue,fc=fc, fdr=ifdr)
    ret = list(x= xmid, y=ymid, fdr.xy=fdr, out=out)
    return(ret)
}

# from OCplus
MidBreaks <- function (x, nr) 
{
    mids = seq(min(x), max(x), length = nr - 1)
    d = mids[2] - mids[1]
    breaks = c(mids - d/2, mids[nr - 1] + d/2)
    breaks
}

## adapted from OCplus simul2d.direct, but with a modification on n==0
## smoothing y/n= proportion of nulls
## NOTE: if n==0 set y/n=1  (all null, so fdr=1)
##
smooth2d.yn<- 
function (y, n, df = 100, err = 0.01, edge.count = 3, mid.start = 6) 
{
    nx = nrow(y)
    ny = ncol(y)
    if (nx < mid.start | edge.count > ny) 
        stop("Grid size too small")
    ## n = ifelse(n <= 1, 1, n)  ## wrong default here!
    edge = 1:edge.count
    mid = mid.start:(nx - mid.start + 1)
    y[mid, edge] = ifelse(n[mid, edge] == 1, 1, y[mid, edge])
    p = y/n
      p = ifelse(n<1, 1, p) ## if n==0 set p=1  (results= null)
    sv2 = df2var(nx, df = df)
    fit = smooth2d.basic(p, sv2 = sv2, err = err)
    list(fit = fit, df = df, sv2 = sv2)
}

## from OCplus
approx2d<- 
function (x, y, z, xout, yout, ...) 
{
    n = length(x)
    m = length(y)
    no = length(xout)
    zout = rep(NA, no)
    for (i in 2:n) {
        for (j in 2:m) {
            ndx = x[i - 1] <= xout & xout <= x[i] & y[j - 1] <= 
                yout & yout <= y[j]
            lower = z[i - 1, j - 1] + (xout[ndx] - x[i - 1]) * 
                (z[i, j - 1] - z[i - 1, j - 1])/(x[i] - x[i - 
                1])
            upper = z[i - 1, j] + (xout[ndx] - x[i - 1]) * (z[i, 
                j] - z[i - 1, j])/(x[i] - x[i - 1])
            zout[ndx] = lower + (yout[ndx] - y[j - 1]) * (upper - 
                lower)/(y[j] - y[j - 1])
        }
    }
    undef = which(is.na(zout))
    if (length(undef > 0)) {
        gg = as.matrix(expand.grid(x, y))
        for (i in undef) {
            d2 = (xout[i] - gg[, 1])^2 + (yout[i] - gg[, 2])
            nn = which.min(d2)
            zout[i] = z[nn]
        }
    }
    zout
}

smooth2d.basic <- function (y, sv2 = 0.01, se2 = 1, err = 0.01) 
{
    W = 1/se2
    nx = nrow(y)
    ny = ncol(y)
    Y = c(y)
    R = rmatrix(nx, ny)
    d = R$d/sv2 + W
    xx = R$x
    ybar = mean(Y)
    B = W * (Y - ybar)
    v = gausei(d, xx, B, sv2, err = err, maxiter = 100)
    est = ybar + v
    matrix(est, ncol = ny)
}

gausei <- function (d, xx, B, sv2, err = 0.01, maxiter = 10) 
{
    ng = length(d)
    xx[xx < 1] = ng + 1
    v = c(B/(d + 1e-06), 0)
    vold = 3 * v
    iter = 1
    while ((sd(vold - v, na.rm = TRUE)/sd(v, na.rm = TRUE) > 
        err) & (iter < maxiter)) {
        vold = v
        vmat = matrix(v[xx], ncol = 4)
        v[1:ng] = (B + c(vmat %*% rep(1, 4))/sv2)/d
        iter = iter + 1
    }
    v[1:ng]
}

rmatrix <- function (nx, ny) 
{
    xymat = matrix(0, nrow = nx, ncol = ny)
    grid = c((row(xymat) - 1) * 1000 + col(xymat) - 1)
    neighbor = c(grid + 1000, grid - 1000, grid + 1, grid - 1)
    loc = match(neighbor, grid, 0)
    loc = matrix(loc, ncol = 4)
    num = c((loc > 0) %*% rep(1, 4))
    list(x = loc, d = num)
}

df2var <- function (nx, df = nx, lower = 1e-04, upper = 1000) 
{
    ff = function(x) dgfree(nx, x) - df
    out = uniroot(ff, lower = lower, upper = upper)
    return(out$root)
}

dgfree <- function (nx, sv2 = 1, ny = nx) 
{
    imp = matrix(0, nrow = nx, ncol = ny)
    imp[nx/2, ny/2] = 1
    imp.resp = smooth2d.basic(imp, sv2 = sv2)
    df = nx * ny * imp.resp[nx/2, ny/2]
    return(df)
}

## OC Curve: true= 0-1 value, pred = probability of true
## extra = extra condition such as VAR_T<0.5
OC = function(true,pred, extra=TRUE, prob=seq(0.01,.99,by=0.01)){
  CUT = SENS= FDR=NULL
  for (pr in prob){
    a = table(true, (pred>pr & extra))
    if (min(dim(a))==1) next
    sens = a[2,2]/(a[2,1]+ a[2,2])
    fdr = a[1,2]/(a[1,2]+a[2,2])
    if ( (fdr-sens)>0 ) next
    SENS = c(SENS, sens)
    FDR = c(FDR,fdr)
    CUT = c(CUT, pr)
  }
  return(list(x=FDR, y=SENS, cut=CUT))
}



